#include "Camera.h"

#include <cmath>
#include <iostream>

Camera::Camera(double fku, double fkv, double u0, double v0) {
	K = cv::Mat::eye(3, 3, CV_64F);
	K.at<double>(0, 0) = fku;
	K.at<double>(1, 1) = fkv;
	K.at<double>(0, 2) = u0;
	K.at<double>(1, 2) = v0;

	R = cv::Mat::eye(3, 3, CV_64F);
	T = cv::Mat::zeros(3, 1, CV_64F);
};

void Camera::setExtrinsics(double phi, double theta, double r) {
	// Spherical -> Cartesian
	double tx = cos(theta) * sin(phi);
	double ty = sin(theta) * sin(phi);
	double tz = cos(phi);

	T.at<double>(0, 0) = -tx * r;
	T.at<double>(1, 0) = -ty * r;
	T.at<double>(2, 0) = -tz * r;

	// Always look at origin
	cv::Point3d Z(-tx, -ty, -tz);
	Z /= cv::norm(Z);


	// X points to the right
	cv::Point3d X(-ty, tx, 0.0f);
	X /= cv::norm(X);

	// If we go into the negative octants, we have to rotate X in the other direction
	// Be careful of negative modulo
	int HowManyPi = std::abs(floor(phi / CV_PI));
	if (HowManyPi % 2 == 1) {
		X *= -1;
	}

	// XYZ are orthogonal and make a right-handed system
	cv::Point3d Y = Z.cross(X);

	R.at<double>(0, 0) = X.x;
	R.at<double>(0, 1) = X.y;
	R.at<double>(0, 2) = X.z;

	R.at<double>(1, 0) = Y.x;
	R.at<double>(1, 1) = Y.y;
	R.at<double>(1, 2) = Y.z;

	R.at<double>(2, 0) = Z.x;
	R.at<double>(2, 1) = Z.y;
	R.at<double>(2, 2) = Z.z;
}